@@ -350,11 +350,9 @@ class Player(GObject.GObject):
 
         self.titleLabel.set_label(AlbumArtCache.get_media_title(media))
 
-        try:
-            artist = media.get_artist()
-            assert artist is not None
-            self.artistLabel.set_label(artist)
-        except:
+        if media.get_artist() is not None:
+            self.artistLabel.set_label(media.get_artist())
+        else:
             self.artistLabel.set_label(_("Unknown Artist"))
 
         url = media.get_url()
@@ -377,11 +375,11 @@ class Player(GObject.GObject):
         if self.player.get_state(1)[1] != Gst.State.PAUSED:
             self.stop()
 
-            media = self.get_current_media()
-            if not media:
-                return
+        media = self.get_current_media()
+        if not media:
+            return
 
-            self.load(media)
+        self.load(media)
 
         self.player.set_state(Gst.State.PLAYING)
         self._update_position_callback()
@@ -664,4 +662,5 @@ class SelectionToolbar():
         self._ui.add_from_resource('/org/gnome/Music/SelectionToolbar.ui')
         self.eventbox = self._ui.get_object('eventbox1')
         self._add_to_playlist_button = self._ui.get_object('button1')
+        self._remove_from_playlist_button = self._ui.get_object('button2')
         self.eventbox.set_visible(False)
