@@ -48,7 +48,8 @@ class ToolbarState:
 class Toolbar(GObject.GObject):
 
     __gsignals__ = {
-        'state-changed': (GObject.SIGNAL_RUN_FIRST, None, ())
+        'state-changed': (GObject.SIGNAL_RUN_FIRST, None, ()),
+        'selection-mode-changed': (GObject.SIGNAL_RUN_FIRST, None, ()),
     }
     _selectionMode = False
     _maximized = False
@@ -56,6 +57,7 @@ class Toolbar(GObject.GObject):
     def __init__(self):
         GObject.GObject.__init__(self)
         self._stack_switcher = StackSwitcher(margin_top=2, margin_bottom=2)
+        self._stack_switcher.show()
         self._ui = Gtk.Builder()
         self._ui.add_from_resource('/org/gnome/Music/headerbar.ui')
         self.header_bar = self._ui.get_object('header-bar')
@@ -68,10 +70,23 @@ class Toolbar(GObject.GObject):
         self._close_button = self._ui.get_object('close-button')
         self._selection_menu = self._ui.get_object('selection-menu')
         self._selection_menu_button = self._ui.get_object('selection-menu-button')
+        self._selection_menu_label = self._ui.get_object('selection-menu-button-label')
         self._selection_menu_button.set_relief(Gtk.ReliefStyle.NONE)
         if Gtk.Widget.get_default_direction() is Gtk.TextDirection.RTL:
             _back_button_image = self._ui.get_object('back-button-image')
             _back_button_image.set_property('icon-name', 'go-previous-rtl-symbolic')
+        if Gtk.get_minor_version() >= 11:
+            self.header_bar.remove(self._close_button)
+            self.header_bar.remove(self._close_separator)
+            self.header_bar.remove(self._select_button)
+            self.header_bar.remove(self._cancel_button)
+            self.header_bar.remove(self._search_button)
+
+            self.header_bar.pack_end(self._close_button)
+            self.header_bar.pack_end(self._close_separator)
+            self.header_bar.pack_end(self._select_button)
+            self.header_bar.pack_end(self._cancel_button)
+            self.header_bar.pack_end(self._search_button)
         self._back_button.connect('clicked', self.on_back_button_clicked)
         self._close_button.connect('clicked', self._close_button_clicked)
         if Gtk.get_minor_version() <= 8:
@@ -116,6 +131,7 @@ class Toolbar(GObject.GObject):
             self._select_button.set_active(False)
             self._select_button.show()
             self._cancel_button.hide()
+        self.emit('selection-mode-changed')
         self._update()
 
     def on_back_button_clicked(self, widget):
